import os
import sys
import hashlib
import click
import openai
from .chat import Tokenizer


@click.command()
@click.option("--model", "-m", default="gpt-3.5-turbo", help="Model to use for translation.")
@click.option("--temperature", "-t", default=0.7, type=float, help="Temperature for sampling.")
@click.option("--max_tokens", "-M", default=1024, type=int, help="Maximum number of tokens in the output.")
@click.option("--chunk_size", "-C", default=2048, type=int, help="Standerd input split token size")
@click.option("--overwrite", "-w", is_flag=True, help="Overwrite cached result.")
@click.option("--verbose", "-v", is_flag=True, help="Display cost information.")
@click.argument("prompt", type=click.STRING, default="", required=False)
def main(model, temperature, max_tokens, chunk_size, overwrite, verbose, prompt):
    openai.api_key = os.environ["OPENAI_API_KEY"]
    tokenizer = Tokenizer(model)
    if not openai.api_key:
        print("環境変数にOpenAI API キーが設定されていません。")
        return

    contents = []
    std_in = ""
    if not sys.stdin.isatty():
        std_in = sys.stdin.read()
        prompt_token_size = tokenizer.calc_token(prompt + "\n")
        chunk_size = chunk_size - prompt_token_size
        contents = [f"{prompt}\n{content}" for content in tokenizer.split_token(std_in, chunk_size)]
    else:
        contents.append(prompt)

    if prompt == "" and std_in == "":
        print("プロンプトが空です")
        return

    price = 0
    cache_dir = os.path.join(os.path.expanduser("~"), ".cache", "gptutil")
    os.makedirs(cache_dir, exist_ok=True)

    for content in contents:
        prompt_hash = hashlib.sha256(content.encode("utf-8")).hexdigest()
        cache_file_path = os.path.join(cache_dir, f"{prompt_hash}.txt")
        if not overwrite and os.path.exists(cache_file_path):
            with open(cache_file_path, "r") as cache_file:
                cache = cache_file.read()
                print(cache)
                continue
        else:
            if verbose:
                price += tokenizer.calc_price(content)
                print(f"token: {tokenizer.calc_token(content)} price: {price} USD")

        with open(cache_file_path, "w") as cache_file:
            for resp in openai.ChatCompletion.create(
                model=model,
                messages=[
                    {"role": "user", "content": content},
                ],
                temperature=temperature,
                max_tokens=max_tokens,
                stream=True,
            ):
                token = resp["choices"][0]["delta"].get("content", "")
                cache_file.write(token)
                sys.stdout.write(token)
                sys.stdout.flush()


if __name__ == "__main__":
    main()



import os
import sys
import requests
import click
import openai

PRICE_PER_IMAGE = {
    "1024x1024": 0.020,
    "512x512": 0.018,
    "256x256": 0.016,
}


@click.command()
@click.option("--prompt", "-p", default="", help="Prompt to generate image.")
@click.option("-v", "--verbose", is_flag=True, help="Show Image URL and cost.")
@click.option(
    "--size",
    "-s",
    type=click.Choice(["1024x1024", "512x512", "256x256"], case_sensitive=False),
    default="256x256",
    help="Image resolution.",
)
@click.option("--n", type=int, default=1, help="Number of images to generate (1-10).")
@click.argument("output", type=click.Path())
def main(prompt, verbose, size, n, output):
    if n < 1 or n > 10:
        print("画像の枚数は1から10の範囲で指定してください。")
        return

    openai.api_key = os.getenv("OPENAI_API_KEY")
    if not openai.api_key:
        print("環境変数にOpenAI API キーが設定されていません。")
        return

    if not sys.stdin.isatty():
        content = sys.stdin.read()
        prompt = f"{prompt}\n{content}"

    response = openai.Image.create(prompt=prompt, n=n, size=size)

    image_data = response["data"]
    for i, data in enumerate(image_data):
        image_url = data["url"]
        if verbose:
            print(f"Image {i + 1} URL: {image_url}")

        output_file = f"{os.path.splitext(output)[0]}_{i + 1}{os.path.splitext(output)[1]}"
        save_image_from_url(image_url, output_file)
        print(f"画像 {i + 1} が {output_file} に保存されました。")

    if verbose:
        cost_per_image = PRICE_PER_IMAGE[size]
        total_cost = cost_per_image * n
        print(f"合計コスト: ${total_cost:.2f}")


def save_image_from_url(url, file_path):
    response = requests.get(url, stream=True)
    response.raise_for_status()
    with open(file_path, "wb") as file:
        for chunk in response.iter_content(chunk_size=8192):
            file.write(chunk)


if __name__ == "__main__":
    main()



import re
import os
import tempfile
import subprocess

class PythonAgent:
    def extract_code_block(self, text):
        return re.findall(r"```python\n(.*?)\n```", text, re.DOTALL)

    def execute_code_block(self, code_block):
        result = ""
        # ランダムなファイル名で一時ファイルを作成
        with tempfile.NamedTemporaryFile(mode='w', delete=False) as f:
            filename = f.name
            # 一時ファイルにコードブロックを書き込む
            f.write(code_block)
        # pythonコマンドから一時ファイルを実行し、標準出力を受け取る
        result = subprocess.check_output(["python", filename]).decode().strip()
        # 一時ファイルを削除する
        os.remove(filename)
        return f"# 実行結果\n```\n{result}\n```"

    def run(self, instructions):
        result = ""
        # markdownのコードブロックを抽出し、それぞれを実行する
        for code_block in self.extract_code_block(instructions):
            # 実行結果を取得してresultに追加する
            result += self.execute_code_block(code_block) + "\n"
        return result.strip()



import sys
import tiktoken
from langchain.prompts import (
    ChatPromptTemplate,
    MessagesPlaceholder,
    SystemMessagePromptTemplate,
    HumanMessagePromptTemplate,
)
from langchain.schema import (
    AIMessage,
    ChatMessage,
    HumanMessage,
    SystemMessage,
)
from langchain.callbacks.base import CallbackManager
from langchain.callbacks.streaming_stdout import StreamingStdOutCallbackHandler
from langchain.chat_models import ChatOpenAI
from langchain.chains import ConversationChain
from langchain.memory import (
    ConversationBufferMemory,
    ConversationSummaryMemory,
    ConversationSummaryBufferMemory,
    ConversationBufferWindowMemory,
    ConversationEntityMemory,
    ConversationKGMemory,
)

# ConversationBufferMemory: 単純に会話記録を保持し、プロンプトに過去会話として入れ込むメモリです
# ConversationSummaryMemory: 会話の要約を保存するメモリです
# ConversationEntityMemory: 会話中の特定の事物にかんして保持するためのメモリです
#                           欠点としては、固有名詞抽出を行っているので表記ゆれに弱いことです
BASE_PROMPT = """
Context:
{entities}
Last line:
{input}
"""


class Tokenizer:
    def __init__(self, model="gpt-3.5-turbo"):
        self.model = model
        self.encoding = tiktoken.encoding_for_model(model)

    def split_token(self, text, chunk_size=4096):
        tokens = self.encoding.encode(text, allowed_special="all")
        chunks = []
        start = 0
        while start < len(tokens):
            end = min(start + chunk_size, len(tokens))
            chunk = self.encoding.decode(tokens[start:end])
            chunks.append(chunk)
            start = end
        return chunks

    def calc_token(self, text):
        return len(self.encoding.encode(text, allowed_special="all"))

    def calc_price(self, text):
        # https://openai.com/pricing
        model_price = {
            "ada": 0.0000004,
            "text-ada-001": 0.0000004,
            "babbage": 0.0000005,
            "text-babbage-001": 0.0000005,
            "curie": 0.000002,
            "text-curie-001": 0.000002,
            "davinci": 0.00002,
            "text-davinci-001": 0.00002,
            "code-davinci-002": 0.00002,
            "text-davinci-002": 0.00002,
            "text-davinci-003": 0.00002,
            "gpt-3.5-turbo": 0.000002,
            "gpt-3.5-turbo-0301": 0.000002,
            "gpt-4": 0.00003,
            "gpt-4-0314": 0.00003,
            "gpt-4-32K": 0.00006,
            "gpt-4-32K-0314": 0.00006,
        }
        token_count = self.calc_token(text)
        return token_count * model_price[model]


class Chat:
    def __init__(self, model="gpt-3.5-turbo", temperature=0.5, max_tokens=1024):
        self.model = model
        self.temperature = temperature
        self.max_tokens = max_tokens
        self.log_file = None
        self.memory = None
        self.reset()

    def reset(self, prompt=None):
        # chatプロンプトテンプレートの準備
        messages = []
        if prompt is not None:
            messages.append(SystemMessagePromptTemplate.from_template(prompt))
        messages.append(MessagesPlaceholder(variable_name="history"))
        messages.append(HumanMessagePromptTemplate.from_template("{input}"))
        # messages.append(HumanMessagePromptTemplate.from_template(BASE_PROMPT))
        prompt_template = ChatPromptTemplate.from_messages(messages)

        # チャットモデルの準備
        llm = ChatOpenAI(
            streaming=True,
            callback_manager=CallbackManager([StreamingStdOutCallbackHandler()]),
            verbose=True,
            temperature=self.temperature,
        )
        # メモリの準備
        # self.memory = ConversationBufferMemory(return_messages=True)
        # self.memory = ConversationBufferWindowMemory(return_messages=True)
        # self.memory = ConversationSummaryMemory(llm=llm, return_messages=True)
        self.memory = ConversationSummaryBufferMemory(llm=llm, return_messages=True)
        # self.memory = ConversationEntityMemory(llm=llm, return_messages=True)
        # self.memory = ConversationKGMemory(llm=llm, return_messages=True)
        # 会話チェーンの準備
        self.conversation = ConversationChain(memory=self.memory, prompt=prompt_template, llm=llm)
        self.messages = []

    def set_log_file(self, log_file):
        self.log_file = log_file

    def show_history(self):
        for message in self.messages:
            if message["role"] == "user":
                print("\033[32m" + f"[{message['role']}] {message['content']}" + "\033[0m")
            elif message["role"] == "assistant":
                print("\033[33m" + f"[{message['role']}] {message['content']}" + "\033[0m")
            elif message["role"] == "system":
                print("\033[31m" + f"[{message['role']}] {message['content']}" + "\033[0m")

    def write_log(self):
        if self.log_file is None:
            return
        with open(self.log_file, "a") as f:
            f.write(f"[{self.messages[-2]['role']}] {self.messages[-2]['content']}\n")
            f.write(f"[{self.messages[-1]['role']}] {self.messages[-1]['content']}\n")

    def ask(self, prompt):
        answer = ""
        try:
            answer = self.conversation.predict(input=prompt)
            print("")
            for message in self.memory.chat_memory.messages:
                if isinstance(message, ChatMessage):
                    self.messages.append({"role": message.role, "content": message.content})
                elif isinstance(message, HumanMessage):
                    self.messages.append({"role": "user", "content": message.content})
                elif isinstance(message, AIMessage):
                    self.messages.append({"role": "assistant", "content": message.content})
                elif isinstance(message, SystemMessage):
                    self.messages.append({"role": "system", "content": message.content})
            self.write_log()
        except Exception as e:
            print(e)

        return answer



from setuptools import setup, find_packages

setup(
    name="gptutil",
    version="0.1.3",
    packages=find_packages(),
    install_requires=[
        "networkx",
        "click",
        "tiktoken",
        "langchain",
        "prompt_toolkit"
    ],
    package_data={"gptutil": ["example/*.yaml"]},
    entry_points={
        "console_scripts": [
            "gpt-ask=gptutil.ask:main",
            "gpt-image=gptutil.image:main",
            "gpt-interact=gptutil.interact:main",
        ],
    },
)



import re
import os
import subprocess


class CodeAgent:
    def __init__(self):
        self.environment = os.environ.copy()
        self.current_directory = os.getcwd()
        self.command_history = []

    def extract_code_block(self, text):
        return re.findall(r"```python\n(.*?)\n```", text, re.DOTALL)

    def extract_commands(self, text):
        return re.findall(r"```bash\n(.*?)\n```", text, re.DOTALL)

    def execute_code_block(self, code_block):
        result = ""
        # ランダムなファイル名で一時ファイルを作成
        with tempfile.NamedTemporaryFile(mode='w', delete=False) as f:
            filename = f.name
            # 一時ファイルにコードブロックを書き込む
            f.write(code_block)
        # pythonコマンドから一時ファイルを実行し、標準出力を受け取る
        result = subprocess.check_output(["python", filename]).decode().strip()
        # 一時ファイルを削除する
        os.remove(filename)
        return f"# python 実行結果\n```\n{result}\n```"

    def execute_single_command(self, command, shell):
        command_parts = command.split(" ")
        if command_parts[0] == "cd":
            target_directory = command_parts[1]
            if not os.path.isabs(target_directory):
                target_directory = os.path.join(self.current_directory, target_directory)
            target_directory = os.path.abspath(target_directory)

            if os.path.isdir(target_directory):
                self.current_directory = target_directory
        if command_parts[0] == "export":
            key, value = command_parts[1].split("=")
            self.environment[key] = value

        shell.stdin.write(command + "\n")
        shell.stdin.flush()

        self.command_history.append(command)

    def execute_command_block(self, command_block):
        with subprocess.Popen(
            ["bash"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
            env=self.environment,
            cwd=self.current_directory,
        ) as shell:
            command_lines = command_block.split("\n")
            formatted_commands = "\n".join([f"$ {cmd}" for cmd in command_lines])

            for command in command_lines:
                self.execute_single_command(command, shell)

            stdout, stderr = shell.communicate()
            output = stdout if stdout else stderr

            return f"```\n{formatted_commands}\n```\n#current directory: {self.current_directory}\n#実行結果\n```\n{output}```\n"

    def run(self, instructions):
        result = ""
        try:
            commands = self.extract_commands(instructions)

            # bashのコマンドブロックを抽出し、それぞれを実行する
            for command_block in commands:
                output = self.execute_command_block(command_block)
                result += output

            # pythonのコードブロックを抽出し、それぞれを実行する
            for code_block in self.extract_code_block(instructions):
                result += self.execute_code_block(code_block) + "\n"

        except Exception as e:
            return str(e)

        return result



import os
import re
import sys
import time
import yaml
import click
import gptutil
import subprocess
import pkg_resources
from prompt_toolkit import PromptSession
from prompt_toolkit.history import InMemoryHistory
from prompt_toolkit.history import FileHistory
from prompt_toolkit.completion import WordCompleter
from .chat import Chat, Tokenizer
from .agent import CodeAgent

EXAMPLE_TEMPLATE = example_path = os.path.join(gptutil.__path__[0], "example", "assistant.yaml")


def replace_commands(input_str):
    def replace_env_var(match):
        return os.environ.get(match.group(1), "")

    def replace_command(match):
        try:
            result = subprocess.check_output(match.group(1), shell=True, text=True)
            return result.strip()
        except subprocess.CalledProcessError:
            return ""

    # 環境変数を置き換える
    replaced_str = re.sub(r"\${(\w+)}", replace_env_var, input_str)
    # コマンドを置き換える
    replaced_str = re.sub(r"`(\w+)`", replace_command, replaced_str)
    return replaced_str


class CLIHandler:
    def __init__(self, template):
        self.template = template
        self.assistant_name = None
        self.chat = Chat()
        self.code_agent = CodeAgent()
        self.tokenizer = Tokenizer()
        self.data = {}

    def handle_command(self, assistant_name):
        self.assistant_name = assistant_name
        assistant = self.template[self.assistant_name]

        history_config = assistant.get("history", {})
        if history_config.get("type") == "file":
            history_file = os.path.expanduser(history_config.get("path", "~/.gpt_interact_history"))
            history = FileHistory(history_file)
            chat_log_dir = os.path.expanduser("~/.gpt_history")
            os.makedirs(chat_log_dir, exist_ok=True)
            self.chat.set_log_file(os.path.join(chat_log_dir, f"{assistant_name}.log"))
        else:
            history = InMemoryHistory()

        commands_completer = WordCompleter(
            ["@use", "@history", "@reset", "@params"] + list(self.template.keys()),
            meta_dict={
                "@use": "Switch assistant",
                "@reset": "Reset chat",
                "@history": "Show chat history",
                "@params": "Show params",
            },
        )

        session = PromptSession(history=history, completer=commands_completer, complete_while_typing=True)

        system_prompt = None
        self.data = {}
        for item in assistant["params"]:
            if item["type"] == "once":
                command, new_assistant = self.handle_params(session, self.assistant_name, item)
                if new_assistant:
                    return new_assistant
                self.data[item["name"]] = command
            if item["type"] == "static":
                static_value = replace_commands(item["value"])
                self.data[item["name"]] = static_value
                print(f"[{assistant_name}] {item['name']}: {static_value}")
            system_prompt = assistant.get("system_prompt", "").format(**self.data)

        self.chat.reset(system_prompt)

        while True:
            for item in assistant["params"]:
                if item["type"] == "each":
                    command, new_assistant = self.handle_params(session, self.assistant_name, item)
                    if new_assistant:
                        return new_assistant
                    self.data[item["name"]] = command
            answer = self.chat.ask(assistant["user_prompt"].format(**self.data))
            agent = assistant.get("agent", {})
            if agent.get("name") == "code":
                while True:
                    cmd_result = self.code_agent.run(answer)
                    print("\033[32m" + cmd_result + "\033[0m")
                    if cmd_result == "":
                        cmd_result = replace_commands(agent.get("args", ""))
                    if self.tokenizer.calc_token(cmd_result) > 4000:
                        cmd_result = "実行結果の文字列が多すぎます。grepコマンドなどを使って文字数を減らして下さい。"
                    time.sleep(5)
                    answer = self.chat.ask(cmd_result)

    def handle_at_command(self, command):
        if command.startswith("@use "):
            new_assistant = command[5:].strip()
            if new_assistant in self.template:
                return command, new_assistant
        elif command.startswith("@reset"):
            self.chat.reset()
            return command, self.assistant_name
        elif command.startswith("@history"):
            self.chat.show_history()
            return None, None
        elif command.startswith("@params"):
            for key in self.data.keys():
                value = self.data[key]
                print(f"{key}: {value}")
            return None, None
        else:
            print(f"{command} not found")
            return None, None
        return command, None

    def handle_params(self, session, assistant_name, item):
        while True:
            command = session.prompt(f"[{assistant_name}] {item['value']}: ")
            command = item.get("default") if command is None else command
            if command.startswith("@"):
                command, new_assistant = self.handle_at_command(command)
                if new_assistant:
                    return None, new_assistant
            if command is None:
                continue
            if command.lower() == "exit":
                sys.exit()
            if command.startswith("!"):
                command = command[1:].strip()
                if command == "":
                    command = os.environ.get("SHELL", "bash")
                os.system(command)
            else:
                return command, None


@click.command()
@click.option(
    "-t",
    "--template",
    "template_file",
    type=click.File("r"),
    default=EXAMPLE_TEMPLATE,
    help="Template file in yaml format",
)
@click.option("-n", "--name", "assistant_name", type=str, help="Name of assistant")
def main(template_file, assistant_name):
    template = yaml.safe_load(template_file)
    handler = CLIHandler(template)
    while True:
        assistant_name = handler.handle_command(assistant_name)


if __name__ == "__main__":
    main()
